// TEENSY 2 - BRUTBOX
// Control inputs (sensors) with MIDI messages
// 08/06/2015 - http://reso-nance.org

#include <Wire.h>
#include "Adafruit_MPR121.h"
#include <Encoder.h>

// Capacitive apacitive MPR121
// Connection Teensy2 
// USB wiring : d+ : SDA, d- : SCL, SCL = DO, SDA = D1 
Adafruit_MPR121 cap = Adafruit_MPR121();
const int touchNb = 12; // number of touch inputs
const int touchThreshold = 60;
int touchStateCtl[] = {40, 41, 42} ; // touch on/off
int touchState[touchNb]; // state : on/off
int currentTouchValues[touchNb]; 
int lastTouchValues[touchNb];
int touchCtl[] = {60,61,62,63,64,65,66,67,68,69,70,71}; // note in
int touchOn = 0;


// Analog setup
int anaPins[] = {22,11,12,13,14,15,16,17,18,19,20,21}; // analog pins
const int anaNb = 12; // number of inputs
int anaCtl[] = {11,10,9,8,7,6,5,4,3,2,1,0}; // controller in
int anaStateCtl[] = {20, 21} ; // sensor on (20), sensor off (21)
int anaState[anaNb]; // state : on/off
int anaValues[anaNb]; // current analog values
int anaLastValues[anaNb]; // previous analog values


// Encoder setup
Encoder encoderSensor (7, 8);
long encoderOldPosition  = -999;
long encoderNewPosition;
int encoderStateCtl = 13; // receive ctlout 13
int encoderOn = 0;

const int channel = 1;

// Sampling rate
const long interval = 30;
unsigned long currentMillis;
unsigned long previousMillis = 0;


void setup() { 
  // Midi receive : on/off sensors 
  usbMIDI.setHandleControlChange(OnControlChange);
} 

void loop() {
  currentMillis = millis();
  if(currentMillis - previousMillis >= interval) {
      previousMillis = currentMillis;

      // Analog sensors loop
      for (int i = 0; i < anaNb; i++) {
          if(anaState[i] == 1)  { // check first if the sensor is on
              anaValues[i] = (int) analogRead(anaPins[i])  / 8 ;
              Serial.println(anaValues[i]);
              if (anaValues[i] != anaLastValues[i]) {  
                  usbMIDI.sendControlChange(anaCtl[i], anaValues[i], channel);
                  anaLastValues[i] = anaValues[i];
	      }
          }
       }
       
      // Capacitive loop
      
      if(touchOn == 1){
      for (uint8_t i=0; i < touchNb; i++) {
         if(touchState[i] == 1)  { // check first if the sensor is on
          currentTouchValues[i] = cap.filteredData(i);
          if ((currentTouchValues[i] - lastTouchValues[i]) < -touchThreshold) {
            usbMIDI.sendNoteOn(touchCtl[i], 127, channel);
          }
          if ((currentTouchValues[i] - lastTouchValues[i]) > touchThreshold) {
            usbMIDI.sendNoteOn(touchCtl[i], 0, channel);
          }
          lastTouchValues[i] = currentTouchValues[i];
        }
      }
      }
     
     
     // Encoder
     if (encoderOn == 1) {
     encoderNewPosition = encoderSensor.read();
     if (encoderNewPosition != encoderOldPosition) {
      encoderOldPosition = encoderNewPosition;
      usbMIDI.sendControlChange(12, encoderNewPosition, channel);
      }
     }
  }
    
  // Discard incoming MIDI messages.
  while (usbMIDI.read()) {}
}

// Receive Midi Control Change
void OnControlChange(byte channel, byte control, byte value) {
   // Analog sensors on/off
   if (control == anaStateCtl[0]) {anaState[-value+(anaNb-1)] = 0;}
   else if (control == anaStateCtl[1]) {anaState[-value+(anaNb-1)] = 1;}
   else if (control == touchStateCtl[0]) {touchState[value] = 0;}
   else if (control == touchStateCtl[1]) {touchState[value] = 1;}
   else if (control == touchStateCtl[2]) {
     touchOn = value;
     if (touchOn == 1) {
       cap.begin(0x5A);
       for (uint8_t i=0; i<4; i++) {lastTouchValues[i] = cap.filteredData(i); }
     }
   }
   else if (control == encoderStateCtl) {encoderOn = value;}
}
